@@ -6,6 +6,7 @@ import shutil
 import logging
 import inspect
 from glob import glob
+from contextlib import contextmanager
 from subprocess import run, Popen, PIPE, STDOUT
 from importlib import import_module, invalidate_caches
 
@@ -28,6 +29,23 @@ import yaml
 import click
 
 
+@contextmanager
+def gha_group(title):
+    """
+    A context manager which groups all console output
+    when running on GitHub Actions CI.
+    In all other cases, this is a NOOP.
+    """
+    if os.environ.get("GITHUB_ACTIONS") == "true":
+        print(f"::group::{title}")
+        try:
+            yield
+        finally:
+            print("::endgroup::")
+    else:
+        yield
+
+
 class AliasedGroup(click.Group):
     def get_command(self, ctx, cmd_name):
         rv = click.Group.get_command(self, ctx, cmd_name)
@@ -163,6 +181,7 @@ def scan_tasks(config):
             new_tasks[name] = (config_value, task_class)
 
     runnable_tasks = []
+    affected_files = []
     logger.info("Loaded tasks:")
     for name, (enabled, task_class) in new_tasks.items():
         logger.info(
@@ -181,16 +200,29 @@ def scan_tasks(config):
                     )
                     sys.exit(1)
             priority = task_class.PRIORITY if hasattr(task_class, "PRIORITY") else 0
+            for file in (
+                task_class.AFFECTED_FILES
+                if hasattr(task_class, "AFFECTED_FILES")
+                else []
+            ):
+                if file in affected_files:
+                    logger.exception(
+                        "Task '{}' specifies an affected file '{}' which was already specified by another task.".format(
+                            name, file
+                        )
+                    )
+                    sys.exit(1)
+                affected_files.append(file)
             runnable_tasks.append((name, task_class, priority))
 
-    return sorted(runnable_tasks, key=lambda x: x[2], reverse=True)
+    return sorted(runnable_tasks, key=lambda x: x[2], reverse=True), affected_files
 
 
 def validate_config(config):
     if config.get("build", None) is None:
-        config["build"] = {"win": True, "mac": True, "android": True}
-    if config["build"].get("win", None) is None:
-        config["build"]["win"] = True
+        config["build"] = {"pc": True, "mac": True, "android": True}
+    if config["build"].get("pc", None) is None:
+        config["build"]["pc"] = True
     if config["build"].get("mac", None) is None:
         config["build"]["mac"] = True
     if config["build"].get("android", None) is None:
@@ -203,15 +235,6 @@ def validate_config(config):
     if config["renutil"].get("registry", None) is None:
         config["renutil"]["registry"] = None
 
-    if config.get("renotize", None) is None:
-        config["renotize"] = {}
-    for key in ("apple_id", "password", "identity", "bundle"):
-        if config["renotize"].get(key, None) is None:
-            logger.error("'{}' is a required key for 'renotize'!".format(key))
-            sys.exit(1)
-    if config["renotize"].get("altool_extra", None) is None:
-        config["renotize"]["altool_extra"] = ""
-
     if config.get("tasks", None) is None:
         config["tasks"] = {"path": None}
     if config["tasks"].get("path", None) is None:
@@ -265,7 +288,7 @@ def cli(project, output, config, debug):
 
     config = validate_config(config)
 
-    runnable_tasks = scan_tasks(config)
+    runnable_tasks, affected_files = scan_tasks(config)
 
     p = run("renutil --help", capture_output=True, shell=True)
     if not (b"Usage: renutil" in p.stdout and p.returncode == 0):
@@ -319,51 +342,78 @@ def cli(project, output, config, debug):
         [item.strip() for item in output][1].lstrip("Install Location:").strip()
     )
 
+    if affected_files:
+        logger.info(
+            "Found {} affected files requiring backup...".format(len(affected_files))
+        )
+        for file in affected_files:
+            full_path = os.path.join(config["renutil"]["path"], file)
+            if not os.path.isfile(full_path):
+                logger.warning("'{}' could not be found".format(file))
+                continue
+            backup_path = os.path.join(
+                os.path.dirname(full_path),
+                "{}.original".format(os.path.basename(full_path)),
+            )
+            if os.path.isfile(backup_path):
+                logger.info("File '{}' already has a backup, restoring...".format(file))
+                shutil.copyfile(backup_path, full_path)
+                continue
+            logger.info("Backing up '{}'...".format(file))
+            shutil.copyfile(full_path, backup_path)
+
     if runnable_tasks:
-        run_tasks(config, runnable_tasks, stage="pre-build")
+        with gha_group("Pre-Build Tasks"):
+            run_tasks(config, runnable_tasks, stage="pre-build")
 
     if config["build"]["android"]:
-        logger.info("Building Android package")
-        cmd = "renutil {} launch {} android_build \
-        {} assembleRelease --destination {}".format(
-            registry_cmd,
-            config["renutil"]["version"],
-            config["project"],
-            config["output"],
-        )
-        proc = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT)
-        for line in proc.stdout:
-            line = str(line.strip(), "utf-8")
-            if line:
-                logger.debug(line)
+        with gha_group("Build Android"):
+            logger.info("Building Android package")
+            cmd = "renutil {} launch {} -h android_build \
+            {} assembleRelease --destination {}".format(
+                registry_cmd,
+                config["renutil"]["version"],
+                config["project"],
+                config["output"],
+            )
+            proc = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT)
+            for line in proc.stdout:
+                line = str(line.strip(), "utf-8")
+                if line:
+                    logger.debug(line)
 
     platforms_to_build = []
-    if config["build"]["win"]:
-        platforms_to_build.append("win")
+    if config["build"]["pc"]:
+        platforms_to_build.append("pc")
     if config["build"]["mac"]:
         platforms_to_build.append("mac")
-    if len(platforms_to_build) == 1:
-        logger.info("Building {} package".format(platforms_to_build[0]))
-    elif len(platforms_to_build) > 1:
-        logger.info("Building {} packages".format(", ".join(platforms_to_build)))
+
     if platforms_to_build:
-        cmd = "renutil {} launch {} distribute \
-        {} --destination {}".format(
-            registry_cmd,
-            config["renutil"]["version"],
-            config["project"],
-            config["output"],
-        )
-        for package in platforms_to_build:
-            cmd += " --package {}".format(package)
-        proc = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT)
-        for line in proc.stdout:
-            line = str(line.strip(), "utf-8")
-            if line:
-                logger.debug(line)
+        with gha_group(f"Build {', '.join(platforms_to_build)}"):
+            if len(platforms_to_build) == 1:
+                logger.info("Building {} package".format(platforms_to_build[0]))
+            elif len(platforms_to_build) > 1:
+                logger.info(
+                    "Building {} packages".format(", ".join(platforms_to_build))
+                )
+            cmd = "renutil {} launch {} -h distribute \
+            {} --destination {}".format(
+                registry_cmd,
+                config["renutil"]["version"],
+                config["project"],
+                config["output"],
+            )
+            for package in platforms_to_build:
+                cmd += " --package {}".format(package)
+            proc = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT)
+            for line in proc.stdout:
+                line = str(line.strip(), "utf-8")
+                if line:
+                    logger.debug(line)
 
     if runnable_tasks:
-        run_tasks(config, runnable_tasks, stage="post-build")
+        with gha_group("Post-Build Tasks"):
+            run_tasks(config, runnable_tasks, stage="post-build")
 
 
 if __name__ == "__main__":
